import type express from 'express';
import type { IDataObject, ExecutionStatus } from 'n8n-workflow';
import type { User } from '../databases/entities/User';
import type { Role } from '../databases/entities/Role';
import type { WorkflowEntity } from '../databases/entities/WorkflowEntity';
import type { UserManagementMailer } from '../UserManagement/email';
import type { Risk } from '../audit/types';
export type AuthlessRequest<RouteParams = {}, ResponseBody = {}, RequestBody = {}, RequestQuery = {}> = express.Request<RouteParams, ResponseBody, RequestBody, RequestQuery>;
export type AuthenticatedRequest<RouteParams = {}, ResponseBody = {}, RequestBody = {}, RequestQuery = {}> = express.Request<RouteParams, ResponseBody, RequestBody, RequestQuery> & {
    user: User;
    globalMemberRole?: Role;
    mailer?: UserManagementMailer;
};
export type PaginatedRequest = AuthenticatedRequest<{}, {}, {}, {
    limit?: number;
    cursor?: string;
    offset?: number;
    lastId?: string;
}>;
export declare namespace ExecutionRequest {
    type GetAll = AuthenticatedRequest<{}, {}, {}, {
        status?: ExecutionStatus;
        limit?: number;
        cursor?: string;
        offset?: number;
        includeData?: boolean;
        workflowId?: string;
        lastId?: string;
    }>;
    type Get = AuthenticatedRequest<{
        id: string;
    }, {}, {}, {
        includeData?: boolean;
    }>;
    type Delete = Get;
}
export declare namespace CredentialTypeRequest {
    type Get = AuthenticatedRequest<{
        credentialTypeName: string;
    }, {}, {}, {}>;
}
export declare namespace WorkflowRequest {
    type GetAll = AuthenticatedRequest<{}, {}, {}, {
        tags?: string;
        status?: ExecutionStatus;
        limit?: number;
        cursor?: string;
        offset?: number;
        workflowId?: number;
        active: boolean;
    }>;
    type Create = AuthenticatedRequest<{}, {}, WorkflowEntity, {}>;
    type Get = AuthenticatedRequest<{
        id: string;
    }, {}, {}, {}>;
    type Delete = Get;
    type Update = AuthenticatedRequest<{
        id: string;
    }, {}, WorkflowEntity, {}>;
    type Activate = Get;
}
export declare namespace UserRequest {
    type Invite = AuthenticatedRequest<{}, {}, Array<{
        email: string;
    }>>;
    type ResolveSignUp = AuthlessRequest<{}, {}, {}, {
        inviterId?: string;
        inviteeId?: string;
    }>;
    type SignUp = AuthenticatedRequest<{
        id: string;
    }, {
        inviterId?: string;
        inviteeId?: string;
    }>;
    type Delete = AuthenticatedRequest<{
        id: string;
        email: string;
    }, {}, {}, {
        transferId?: string;
        includeRole: boolean;
    }>;
    type Get = AuthenticatedRequest<{
        id: string;
        email: string;
    }, {}, {}, {
        limit?: number;
        offset?: number;
        cursor?: string;
        includeRole?: boolean;
    }>;
    type Reinvite = AuthenticatedRequest<{
        id: string;
    }>;
    type Update = AuthlessRequest<{
        id: string;
    }, {}, {
        inviterId: string;
        firstName: string;
        lastName: string;
        password: string;
    }>;
}
export declare namespace CredentialRequest {
    type Create = AuthenticatedRequest<{}, {}, {
        type: string;
        name: string;
        data: IDataObject;
    }, {}>;
}
export type OperationID = 'getUsers' | 'getUser';
type PaginationBase = {
    limit: number;
};
export type PaginationOffsetDecoded = PaginationBase & {
    offset: number;
};
export type PaginationCursorDecoded = PaginationBase & {
    lastId: string;
};
export type OffsetPagination = PaginationBase & {
    offset: number;
    numberOfTotalRecords: number;
};
export type CursorPagination = PaginationBase & {
    lastId: string;
    numberOfNextRecords: number;
};
export interface IRequired {
    required?: string[];
    not?: {
        required?: string[];
    };
}
export interface IDependency {
    if?: {
        properties: {};
    };
    then?: {
        allOf: IRequired[];
    };
    else?: {
        allOf: IRequired[];
    };
}
export interface IJsonSchema {
    additionalProperties: boolean;
    type: 'object';
    properties: {
        [key: string]: {
            type: string;
        };
    };
    allOf?: IDependency[];
    required: string[];
}
export declare class SourceControlPull {
    force?: boolean;
    variables?: {
        [key: string]: string;
    };
}
export declare namespace PublicSourceControlRequest {
    type Pull = AuthenticatedRequest<{}, {}, SourceControlPull, {}>;
}
export declare namespace AuditRequest {
    type Generate = AuthenticatedRequest<{}, {}, {
        additionalOptions?: {
            categories?: Risk.Category[];
            daysAbandonedWorkflow?: number;
        };
    }>;
}
export {};
