"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseTagNames = exports.updateWorkflow = exports.getWorkflowsAndCount = exports.deleteWorkflow = exports.setWorkflowAsInactive = exports.setWorkflowAsActive = exports.createWorkflow = exports.getWorkflowIdsViaTags = exports.getWorkflowById = exports.getSharedWorkflows = exports.getSharedWorkflow = exports.getSharedWorkflowIds = void 0;
const typeorm_1 = require("typeorm");
const intersection_1 = __importDefault(require("lodash/intersection"));
const Db = __importStar(require("../../../../Db"));
const WorkflowEntity_1 = require("../../../../databases/entities/WorkflowEntity");
const SharedWorkflow_1 = require("../../../../databases/entities/SharedWorkflow");
const config_1 = __importDefault(require("../../../../config"));
const tag_service_1 = require("../../../../services/tag.service");
const typedi_1 = __importDefault(require("typedi"));
function insertIf(condition, elements) {
    return condition ? elements : [];
}
async function getSharedWorkflowIds(user) {
    const where = user.globalRole.name === 'owner' ? {} : { userId: user.id };
    const sharedWorkflows = await Db.collections.SharedWorkflow.find({
        where,
        select: ['workflowId'],
    });
    return sharedWorkflows.map(({ workflowId }) => workflowId);
}
exports.getSharedWorkflowIds = getSharedWorkflowIds;
async function getSharedWorkflow(user, workflowId) {
    return Db.collections.SharedWorkflow.findOne({
        where: {
            ...(!user.isOwner && { userId: user.id }),
            ...(workflowId && { workflowId }),
        },
        relations: [...insertIf(!config_1.default.getEnv('workflowTagsDisabled'), ['workflow.tags']), 'workflow'],
    });
}
exports.getSharedWorkflow = getSharedWorkflow;
async function getSharedWorkflows(user, options) {
    return Db.collections.SharedWorkflow.find({
        where: {
            ...(!user.isOwner && { userId: user.id }),
            ...(options.workflowIds && { workflowId: (0, typeorm_1.In)(options.workflowIds) }),
        },
        ...(options.relations && { relations: options.relations }),
    });
}
exports.getSharedWorkflows = getSharedWorkflows;
async function getWorkflowById(id) {
    return Db.collections.Workflow.findOne({
        where: { id },
    });
}
exports.getWorkflowById = getWorkflowById;
async function getWorkflowIdsViaTags(tags) {
    const dbTags = await typedi_1.default.get(tag_service_1.TagService).findMany({
        where: { name: (0, typeorm_1.In)(tags) },
        relations: ['workflows'],
    });
    const workflowIdsPerTag = dbTags.map((tag) => tag.workflows.map((workflow) => workflow.id));
    return (0, intersection_1.default)(...workflowIdsPerTag);
}
exports.getWorkflowIdsViaTags = getWorkflowIdsViaTags;
async function createWorkflow(workflow, user, role) {
    return Db.transaction(async (transactionManager) => {
        const newWorkflow = new WorkflowEntity_1.WorkflowEntity();
        Object.assign(newWorkflow, workflow);
        const savedWorkflow = await transactionManager.save(newWorkflow);
        const newSharedWorkflow = new SharedWorkflow_1.SharedWorkflow();
        Object.assign(newSharedWorkflow, {
            role,
            user,
            workflow: savedWorkflow,
        });
        await transactionManager.save(newSharedWorkflow);
        return savedWorkflow;
    });
}
exports.createWorkflow = createWorkflow;
async function setWorkflowAsActive(workflow) {
    return Db.collections.Workflow.update(workflow.id, { active: true, updatedAt: new Date() });
}
exports.setWorkflowAsActive = setWorkflowAsActive;
async function setWorkflowAsInactive(workflow) {
    return Db.collections.Workflow.update(workflow.id, { active: false, updatedAt: new Date() });
}
exports.setWorkflowAsInactive = setWorkflowAsInactive;
async function deleteWorkflow(workflow) {
    return Db.collections.Workflow.remove(workflow);
}
exports.deleteWorkflow = deleteWorkflow;
async function getWorkflowsAndCount(options) {
    return Db.collections.Workflow.findAndCount(options);
}
exports.getWorkflowsAndCount = getWorkflowsAndCount;
async function updateWorkflow(workflowId, updateData) {
    return Db.collections.Workflow.update(workflowId, updateData);
}
exports.updateWorkflow = updateWorkflow;
function parseTagNames(tags) {
    return tags.split(',').map((tag) => tag.trim());
}
exports.parseTagNames = parseTagNames;
//# sourceMappingURL=workflows.service.js.map