import type { CommitResult, DiffResult, FetchResult, PullResult, PushResult, SimpleGit, StatusResult } from 'simple-git';
import type { SourceControlPreferences } from './types/sourceControlPreferences';
import type { User } from '../../databases/entities/User';
export declare class SourceControlGitService {
    git: SimpleGit | null;
    private gitOptions;
    private preInitCheck;
    initService(options: {
        sourceControlPreferences: SourceControlPreferences;
        gitFolder: string;
        sshFolder: string;
        sshKeyName: string;
    }): Promise<void>;
    resetService(): void;
    private checkRepositorySetup;
    private hasRemote;
    initRepository(sourceControlPreferences: Pick<SourceControlPreferences, 'repositoryUrl' | 'branchName' | 'initRepo'>, user: User): Promise<void>;
    setGitUserDetails(name: string, email: string): Promise<void>;
    getBranches(): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    setBranch(branch: string): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    getCurrentBranch(): Promise<{
        current: string;
        remote: string;
    }>;
    diffRemote(): Promise<DiffResult | undefined>;
    diffLocal(): Promise<DiffResult | undefined>;
    fetch(): Promise<FetchResult>;
    pull(options?: {
        ffOnly: boolean;
    }): Promise<PullResult>;
    push(options?: {
        force: boolean;
        branch: string;
    }): Promise<PushResult>;
    stage(files: Set<string>, deletedFiles?: Set<string>): Promise<string>;
    resetBranch(options?: {
        hard: boolean;
        target: string;
    }): Promise<string>;
    commit(message: string): Promise<CommitResult>;
    status(): Promise<StatusResult>;
}
